@@ -13,7 +13,11 @@
 #include "lib/util_list.h"
 #include "exit_code.h"
 
+struct attrib;
+struct setting_list;
+
 extern int udev_need_settle;
+extern int udev_no_settle;
 
 /* Single key-operator-value entry in a udev rule line.*/
 struct udev_entry_node {
@@ -36,12 +40,18 @@ struct udev_file {
 };
 
 exit_code_t udev_read_file(const char *, struct udev_file **);
+bool udev_file_is_empty(struct udev_file *file);
 void udev_free_file(struct udev_file *);
 void udev_file_print(struct udev_file *);
 
-void udev_get_device_ids(const char *, struct util_list *);
-exit_code_t udev_remove_rule(const char *, const char *);
+void udev_get_device_ids(const char *type, struct util_list *list,
+			 bool autoconf);
+exit_code_t udev_remove_rule(const char *type, const char *id, bool autoconf);
 
 void udev_settle(void);
 
+void udev_add_internal_from_entry(struct setting_list *list,
+				  struct udev_entry_node *entry,
+				  struct attrib **attribs);
+
 #endif /* UDEV_H */
