@@ -15,6 +15,7 @@
 #include "ccwgroup.h"
 #include "device.h"
 #include "devtype.h"
+#include "internal.h"
 #include "misc.h"
 #include "namespace.h"
 #include "nic.h"
@@ -1170,6 +1171,37 @@ static exit_code_t check_ineffective_settings(struct setting_list *list,
 	return rc;
 }
 
+/* Check if a possibly conflicting setting is active in the configuration */
+static bool conflict_setting_active(struct setting *s)
+{
+	enum qeth_attr_group_type t;
+
+	t = get_attr_group_type(s);
+	if (t != group_bridge && t != group_vnicc) {
+		/* Check BridgePort and VNICC attributes only */
+		return false;
+	}
+	if (s->specified) {
+		/* Specified on the command line: We are strict here and do not
+		 * allow to specify VNICC and BridgePort attributes in the same
+		 * command to avoid issues when attributes are enabled/disabled
+		 * in the wrong order. Example: disable VNICC and enable
+		 * BridgePort in the same command would	result in an error
+		 * because BridgePort attributes are set first.
+		 */
+		return true;
+	}
+	if (attrib_match_default(s->attrib, s->value)) {
+		/* Not active if set to default value */
+		return false;
+	}
+	if (s->actual_value && strncmp(s->actual_value, "n/a", 3) == 0) {
+		/* Not active if in n/a state (conflicting attribute set) */
+		return false;
+	}
+	return true;
+}
+
 /* Check if there are conflicting attribute settings */
 static exit_code_t check_conflicting_settings(struct setting_list *list)
 {
@@ -1181,6 +1213,8 @@ static exit_code_t check_conflicting_settings(struct setting_list *list)
 	util_list_iterate(&list->list, s) {
 		if (s->removed)
 			continue;
+		if (!conflict_setting_active(s))
+			continue;
 		t = get_attr_group_type(s);
 		if (t == group_bridge && (!bridge || !bridge->specified))
 			bridge = s;
@@ -1207,7 +1241,7 @@ static exit_code_t check_layer2(struct device *dev, config_t config)
 {
 	struct setting *l;
 	int layer2_detected, layer2_active = -1, layer2_persistent = -1,
-	    layer2_modified = 0;
+	    layer2_autoconf = -1, layer2_modified = 0;
 	exit_code_t rc = EXIT_OK;
 
 	layer2_detected = detect_layer2(dev);
@@ -1231,13 +1265,25 @@ static exit_code_t check_layer2(struct device *dev, config_t config)
 		if (rc)
 			goto out;
 	}
+	l = setting_list_find(dev->autoconf.settings, qeth_attr_layer2.name);
+	if (l) {
+		layer2_autoconf = atoi(l->value);
+		layer2_modified |= l->modified;
+	} else if (SCOPE_AUTOCONF(config)) {
+		rc = generate_layer2("autoconf", dev->autoconf.settings,
+				     &layer2_autoconf, &layer2_modified);
+		if (rc)
+			goto out;
+	}
 
 	/* Check correct layer2 setting. */
 	if (layer2_detected != -1) {
 		if ((SCOPE_ACTIVE(config) && layer2_active != -1 &&
 		     layer2_active != layer2_detected) ||
 		    (SCOPE_PERSISTENT(config) && layer2_persistent != -1 &&
-		     layer2_persistent != layer2_detected)) {
+		     layer2_persistent != layer2_detected) ||
+		    (SCOPE_AUTOCONF(config) && layer2_autoconf != -1 &&
+		     layer2_autoconf != layer2_detected)) {
 			rc = layer2_mismatch(layer2_detected, layer2_modified);
 			if (rc)
 				goto out;
@@ -1256,14 +1302,25 @@ static exit_code_t check_layer2(struct device *dev, config_t config)
 		if (rc)
 			goto out;
 	}
+	if (SCOPE_AUTOCONF(config)) {
+		rc = check_ineffective_settings(dev->autoconf.settings,
+						layer2_autoconf);
+		if (rc)
+			goto out;
+	}
 	/* check for conflicting layer2 attribute groups */
 	if (SCOPE_ACTIVE(config)) {
 		rc = check_conflicting_settings(dev->active.settings);
 		if (rc)
 			goto out;
 	}
-	if (SCOPE_PERSISTENT(config))
+	if (SCOPE_PERSISTENT(config)) {
 		rc = check_conflicting_settings(dev->persistent.settings);
+		if (rc)
+			goto out;
+	}
+	if (SCOPE_AUTOCONF(config))
+		rc = check_conflicting_settings(dev->autoconf.settings);
 
 out:
 	return rc;
@@ -1277,7 +1334,8 @@ static exit_code_t qeth_st_check_pre_configure(struct subtype *st,
 
 	/* No need to check if device is deconfigured. */
 	if ((SCOPE_ACTIVE(config) && dev->active.deconfigured) ||
-	    (SCOPE_PERSISTENT(config) && dev->persistent.deconfigured))
+	    (SCOPE_PERSISTENT(config) && dev->persistent.deconfigured) ||
+	    (SCOPE_AUTOCONF(config) && dev->autoconf.deconfigured))
 		return EXIT_OK;
 
 	rc = check_layer2(dev, config);
@@ -1369,7 +1427,7 @@ static struct ccwgroup_subtype_data qeth_data = {
 	.num_devs	= QETH_NUM_DEVS,
 };
 
-static struct subtype qeth_subtype_qeth = {
+struct subtype qeth_subtype_qeth = {
 	.super		= &ccwgroup_subtype,
 	.devtype	= &qeth_devtype,
 	.name		= "qeth",
@@ -1421,6 +1479,7 @@ static struct subtype qeth_subtype_qeth = {
 		&qeth_attr_vnicc_takeover_learning,
 		&qeth_attr_vnicc_bridge_invisible,
 		&qeth_attr_vnicc_rx_bcast,
+		&internal_attr_early,
 	),
 	.unknown_dev_attribs	= 1,
 	.support_definable	= 1,
