@@ -3,24 +3,25 @@
  *
  * Multi-volume ECKD DASD dump tool
  *
- * Copyright IBM Corp. 2013, 2017
+ * Copyright IBM Corp. 2013, 2018
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
  */
 
+#include "lib/zt_common.h"
+
 #include "eckd2dump.h"
 #include "error.h"
 #include "stage2dump.h"
 
-#define MVDUMP_SIZE		0x3000	/* Size of dump record */
+#define MVDUMP_TOOL_SIZE	0x3000	/* Size of dump record */
 #define MAX_DUMP_VOLUMES	32	/* Up to 32 dump volumes possible */
 
 /*
  * Magic number at start of dump record
  */
-uint64_t magic __attribute__((section(".stage2.head")))
-	= 0x5a4d554c54363405ULL; /* ZMULT64, version 5 */
+uint64_t __section(.stage2.head) magic = 0x584d554c54363401ULL; /* XMULT64, version 1 */
 
 /*
  * Parameter format for ECKD MV dumper (13 bytes):
@@ -59,8 +60,7 @@ struct mvdump_parm_table {
 			(MAX_DUMP_VOLUMES * (sizeof(struct mvdump_param) + 1))];
 } __packed;
 
-static struct mvdump_parm_table mvdump_table
-	__attribute__((section(".eckd2dump_mv.tail")));
+static struct mvdump_parm_table __section(.eckd2dump_mv.tail) mvdump_table;
 
 static int volnr_current;
 
@@ -188,73 +188,117 @@ void dt_device_enable(void)
 	stage2dump_eckd_init();
 }
 
+/*
+ * Check for the volume timestamp and validate the dump signature
+ * before writing a dump.
+ */
+static void check_volume(unsigned long page)
+{
+	struct mvdump_parm_table *mvdump_table_new;
+	struct df_s390_hdr *hdr_new;
+
+	/*
+	 * Check whether parameter table on dump device has a valid
+	 * time stamp. The parameter table is located right behind
+	 * the dump tool, the corresponding block number is:
+	 *   MAGIC_BLK_ECKD + (MVDUMP_TOOL_SIZE / blocksize)
+	 */
+	mvdump_table_new = (void *) page;
+	readblock(DF_S390_MAGIC_BLK_ECKD + m2b(MVDUMP_TOOL_SIZE),
+		  __pa(mvdump_table_new), 1);
+	/*
+	 * Check if time stamps match
+	 */
+	if (mvdump_table.tod != mvdump_table_new->tod)
+		panic(ENOTIME, "Inconsistent time stamps");
+	/*
+	 * Check if dump partition has a valid dump signature.
+	 * Bypass signature check if "--force" had been specified during
+	 * zipl -M.
+	 */
+	hdr_new = (void *) page;
+	if (!parm_tail.mvdump_force) {
+		readblock(device.blk_start, __pa(hdr_new), 1);
+		if (dump_hdr->magic != hdr_new->mvdump_sign)
+			panic(ENOSIGN, "Wrong signature");
+	}
+}
+
+/*
+ * Write the dump header and memory to the current volume and return the next
+ * address to write for the next volume or memory size if the dump ended
+ * on this volume
+ */
+static unsigned long write_volume(unsigned long addr, unsigned long page,
+				  struct df_s390_dump_segm_hdr *dump_segm)
+{
+	unsigned long free_space, blk;
+
+	/*
+	 * Write dump header
+	 */
+	blk = device.blk_start;
+	writeblock(blk, __pa(dump_hdr), m2b(DF_S390_HDR_SIZE), 0);
+	blk += m2b(DF_S390_HDR_SIZE);
+	/*
+	 * The free space left on the volume minus 2 blocks (for the segment
+	 * header and the end marker)
+	 */
+	free_space = b2m(device.blk_end - blk + 1) - b2m(2);
+	memset((void *) page, 0, PAGE_SIZE);
+	/*
+	 * Write dump data
+	 */
+	while (addr < dump_hdr->mem_size) {
+		/*
+		 * Find the next non-zero dump segment with the limit
+		 * of segment length set to the amount of free space left
+		 */
+		addr = find_dump_segment(addr, dump_hdr->mem_size,
+					 ROUND_DOWN(free_space, MIB),
+					 dump_segm);
+		blk = write_dump_segment(blk, dump_segm, page);
+		/* Update free space left on vol */
+		free_space -= dump_segm->len;
+		/* Reserve one block for the next segment header */
+		if (free_space)
+			free_space -= b2m(1);
+		total_dump_size += dump_segm->len;
+		/* Check if no more dump segments follow */
+		if (dump_segm->stop_marker) {
+			/* Write end marker */
+			df_s390_em_page_init(page);
+			writeblock(blk, page, 1, 0);
+			return dump_hdr->mem_size;
+		}
+		/*
+		 * Go to the new volume if not enough space
+		 */
+		if (free_space < MIB)
+			break;
+	}
+	return addr;
+}
+
 /*
  * Dump all memory to multiple DASD partitions
  */
 void dt_dump_mem(void)
 {
-	unsigned long blk, addr, addr_end, blk_num, dev_mem_size, page;
-	struct mvdump_parm_table *mvdump_table_new;
-	struct df_s390_hdr *hdr_new;
+	struct df_s390_dump_segm_hdr *dump_segm;
+	unsigned long addr, page;
 
-	dump_hdr->mvdump_sign = DF_S390_MAGIC;
+	dump_hdr->mvdump_sign = DF_S390_MAGIC_EXT;
 	dump_hdr->mvdump = 1;
 	addr = 0;
-
+	total_dump_size = 0;
 	page = get_zeroed_page();
-	do {
+	dump_segm = (void *)get_zeroed_page();
+
+	while (1) {
 		printf("Dumping to: 0.%x.%04x", device.sid.ssid, device.devno);
-
-		/*
-		 * Check whether parameter table on dump device has a valid
-		 * time stamp. The parameter table is located right behind
-		 * the dump tool, the corresponding block number is:
-		 *   MAGIC_BLOCK_OFFSET + (MVDUMP_TOOL_SIZE / blocksize)
-		 *   So dump tool starts on track 0, block 3
-		 */
-		mvdump_table_new = (void *) page;
-		readblock(3 + MVDUMP_SIZE / 0x1000,
-			  __pa(mvdump_table_new), m2b(0x1000));
-		/*
-		 * Check if time stamps match
-		 */
-		if (mvdump_table.tod != mvdump_table_new->tod)
-			panic(ENOTIME, "Inconsistent time stamps");
-
-		/*
-		 * Check if dump partition has a valid dump signature.
-		 * Bypass signature check if "--force" had been specified during
-		 * zipl -M.
-		 */
-		hdr_new = (void *) page;
-		if (!parm_tail.mvdump_force) {
-			readblock(device.blk_start, __pa(hdr_new), 1);
-			if (dump_hdr->magic != hdr_new->mvdump_sign)
-				panic(ENOSIGN, "Wrong signature");
-		}
-
-		/*
-		 * Write dump header
-		 */
-		blk = device.blk_start;
-		writeblock(blk, __pa(dump_hdr), m2b(DF_S390_HDR_SIZE), 0);
-		blk += m2b(DF_S390_HDR_SIZE);
-
-		dev_mem_size = b2m(device.blk_end - blk + 1);
-		addr_end = MIN(dump_hdr->mem_size, addr + dev_mem_size);
-
-		/*
-		 * Write memory
-		 */
-		memset((void *) page, 0, PAGE_SIZE);
-		while (addr < addr_end) {
-			blk_num = MIN(eckd_blk_max, device.blk_end - blk + 1);
-			blk_num = MIN(m2b(addr_end - addr), blk_num);
-			writeblock(blk, addr, blk_num, page);
-			progress_print(addr);
-			blk += blk_num;
-			addr += b2m(blk_num);
-		}
+		check_volume(page);
+		addr = write_volume(addr, page, dump_segm);
 		if (addr == dump_hdr->mem_size)
 			break;
 		/*
@@ -267,12 +311,8 @@ void dt_dump_mem(void)
 		dt_device_parm_setup();
 		set_device(device.sid, DISABLED);
 		dt_device_enable();
-	} while (1);
+	}
 	progress_print(addr);
-	/*
-	 * Write end marker
-	 */
-	df_s390_em_page_init(page);
-	writeblock(blk, page, 1, 0);
 	free_page(page);
+	free_page(__pa(dump_segm));
 }
