@@ -11,6 +11,8 @@
 
 #include <stdarg.h>
 
+#include "lib/zt_common.h"
+
 #include "error.h"
 #include "libc.h"
 #include "sclp.h"
@@ -343,7 +345,7 @@ void pgm_check_handler_fn(void)
 	libc_stop(psw_old->addr);
 }
 
-__attribute__ ((noinline)) void load_wait_psw(uint64_t psw_mask, struct psw_t *psw)
+void __noinline load_wait_psw(uint64_t psw_mask, struct psw_t *psw)
 {
 	struct psw_t wait_psw = { .mask = psw_mask, .addr = 0 };
 	struct psw_t old_psw, *wait_psw_ptr = &wait_psw;
@@ -383,7 +385,7 @@ void initialize(void)
 /*
  * Load disabled wait PSW with reason code in address field
  */
-void libc_stop(unsigned long reason)
+void __noreturn libc_stop(unsigned long reason)
 {
 	struct psw_t psw;
 
