@@ -11,6 +11,8 @@
 #ifndef LIBC_H
 #define LIBC_H
 
+#include <stdint.h>
+
 #define NULL ((void *) 0)
 
 #define EPERM		 1	/* Operation not permitted */
@@ -41,11 +43,6 @@
 
 #define MIB	(1024ULL * 1024)
 
-typedef unsigned long long uint64_t;
-typedef unsigned int uint32_t;
-typedef unsigned short uint16_t;
-typedef unsigned char uint8_t;
-
 void printf(const char *, ...);
 void sprintf(char *, const char *, ...);
 void *memcpy(void *, const void *, unsigned long);
@@ -60,7 +57,7 @@ char *strcpy(char *, const char *);
 unsigned long get_zeroed_page(void);
 void free_page(unsigned long);
 void initialize(void);
-void libc_stop(unsigned long) __attribute__((noreturn));
+void libc_stop(unsigned long);
 void start(void);
 void pgm_check_handler(void);
 void pgm_check_handler_fn(void);
