@@ -3,7 +3,7 @@
  *
  * Common functions for stand-alone dump tools
  *
- * Copyright IBM Corp. 2013, 2017
+ * Copyright IBM Corp. 2013, 2018
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -11,6 +11,8 @@
 
 #include <stdarg.h>
 
+#include "lib/zt_common.h"
+
 #include "error.h"
 #include "sclp.h"
 #include "stage2dump.h"
@@ -22,8 +24,6 @@
  * Static globals
  */
 static uint8_t machine_has_vx;
-static unsigned long progress_next_addr;
-static unsigned long progress_inc = 0x20000000; /* Issue message each 512M */
 
 /*
  * IPL info in lowcore
@@ -36,8 +36,7 @@ struct ipib_info {
 /*
  * Tail parameters
  */
-struct stage2dump_parm_tail parm_tail
-	__attribute__ ((section(".stage2dump.tail"))) = {
+struct stage2dump_parm_tail parm_tail = {
 	.mem_upper_limit = 0xffffffffffffffffULL,
 };
 
@@ -45,6 +44,7 @@ struct stage2dump_parm_tail parm_tail
  * Globals
  */
 struct df_s390_hdr *dump_hdr;
+unsigned long total_dump_size;
 
 /*
  * Init dumper: Allocate standard pages, set timers
@@ -61,29 +61,132 @@ static void init_early(void)
 		machine_has_vx = 1;
 }
 
-/*
-
- * Init dump progress messages to sclp console
- *
- * 8 messages if the memory is 4G or less, otherwise one message for each
- * 512M chunk of dumped memory
- */
-static void progress_init(void)
-{
-	if (dump_hdr->mem_size <= (4 * 1024 * 1024 * 1024ULL))
-		progress_inc = dump_hdr->mem_size >> 3;
-	progress_next_addr = progress_inc;
-}
-
 /*
  * Print progress message
  */
 void progress_print(unsigned long addr)
 {
-	if (addr < progress_next_addr && addr != dump_hdr->mem_size)
+	/* Print a message approximately every 5 sec */
+	static unsigned long delta = 5 * (1UL << 32);
+	static unsigned long next;
+	unsigned long time = get_tod_clock();
+
+	if (next == 0) {
+		next = time + delta;
 		return;
-	printf("%08lu / %08lu MB", addr >> 20, dump_hdr->mem_size >> 20);
-	progress_next_addr += progress_inc;
+	}
+	if (time < next && addr != dump_hdr->mem_size)
+		return;
+
+	printf("%08lu / %08lu MB (Dump file size %08lu MB)", addr >> 20,
+	       dump_hdr->mem_size >> 20, total_dump_size >> 20);
+	next = time + delta;
+}
+
+/*
+ * Check if one MB memory after addr is zero
+ */
+static int __is_zero_mb(unsigned long addr)
+{
+	register unsigned long _addr1 asm("2") = (unsigned long) addr;
+	register unsigned long _len1  asm("3") = (unsigned long) MIB;
+	register unsigned long _addr2 asm("4") = (unsigned long) addr;
+	register unsigned long _len2  asm("5") = (unsigned long) 0;
+	unsigned int ipm;
+
+	asm volatile(
+		"0:     clcle   %1,%3,0(0)\n"
+		"       jo      0b\n"
+		"       ipm     %0\n"
+		"       srl     %0,28\n"
+		: "=d" (ipm), "+d" (_addr1), "+d" (_len1), "+d" (_addr2), "+d" (_len2)
+		:
+		: "cc");
+	return !ipm;
+}
+
+/*
+ * Check if the megabyte contains all zeroes.
+ */
+int is_zero_mb(unsigned long addr)
+{
+	if (!page_is_valid(addr))
+		return 1;
+	if (__is_zero_mb(addr))
+		return 1;
+	return 0;
+}
+
+/*
+ * Find next non-zero megabyte in the address range
+ */
+static unsigned long find_next_non_zero(unsigned long start, unsigned long end)
+{
+	while (start < end) {
+		if (!is_zero_mb(start))
+			break;
+		start += MIB;
+	}
+	if (start > end)
+		start = end;
+	return start;
+}
+
+/*
+ * Find next zero megabyte in the address range
+ */
+static unsigned long find_next_zero(unsigned long start, unsigned long end)
+{
+	while (start < end) {
+		if (is_zero_mb(start))
+			break;
+		start += MIB;
+	}
+	if (start > end)
+		start = end;
+	return start;
+}
+
+/*
+ * Find first continuous set of non-zero megabytes in the specified
+ * address range and update *dump_segm structure accordingly. The segment's
+ * length is truncated to max_len if required. Afterwards, look for the next
+ * non-zero megabyte. Return the address of the next non-zero megabyte or zero
+ * if no more follow.
+ */
+unsigned long find_dump_segment(unsigned long start, unsigned long end,
+				unsigned long max_len,
+				struct df_s390_dump_segm_hdr *dump_segm)
+{
+	unsigned long addr, limit;
+
+	memset(dump_segm, 0, sizeof(*dump_segm));
+	/* Check the input values for MB alingment */
+	if (!IS_ALIGNED(start, MIB) || !IS_ALIGNED(max_len, MIB))
+		panic(EINTERNAL, "start or max_len not MB aligned");
+	/* Search for dump segment start */
+	addr = find_next_non_zero(start, end);
+	if (addr == end)
+		goto out;
+	dump_segm->start = addr;
+	/* Search for dump segment end */
+	limit = end;
+	if (max_len)
+		limit = MIN(addr + max_len, end);
+	addr = find_next_zero(dump_segm->start, limit);
+	dump_segm->len = addr - dump_segm->start;
+	/* Search for next dump segment start */
+	addr = find_next_non_zero(addr, end);
+	/* Return the start address of the next segment */
+	if (addr < end)
+		return addr;
+out:
+	/*
+	 * Set the stop-marker indicating no more non-zero
+	 * segments in the address range
+	 */
+	dump_segm->stop_marker = -1UL;
+	return 0;
 }
 
 /*
@@ -113,11 +216,11 @@ static void df_s390_dump_init(void)
 {
 	struct df_s390_hdr *dh = dump_hdr;
 
-	dh->magic = DF_S390_MAGIC;
+	dh->magic = DF_S390_MAGIC_EXT;
 	dh->hdr_size = DF_S390_HDR_SIZE;
 	dh->page_size = PAGE_SIZE;
 	dh->dump_level = 4;
-	dh->version = 5;
+	dh->version = 1;
 	dh->mem_start = 0;
 	dh->arch = DF_S390_ARCH_64;
 	dh->build_arch = DF_S390_ARCH_64;
@@ -327,7 +430,6 @@ void start(void)
 	printf("Dumping 64 bit OS");
 	mem_and_cpu_init();
 	store_status();
-	progress_init();
 	dt_dump_mem();
 	printf("Dump successful");
 	dump_exit(0);
