@@ -15,12 +15,12 @@
 #include "libc.h"
 #include "s390.h"
 
-#define IPL_DEVICE		 0x10404
-#define INITRD_START		 0x10408
-#define INITRD_SIZE		 0x10410
-#define OLDMEM_BASE		 0x10418
-#define OLDMEM_SIZE		 0x10420
-#define COMMAND_LINE		 0x10480
+#define IPL_DEVICE		 0x10404UL
+#define INITRD_START		 0x10408UL
+#define INITRD_SIZE		 0x10410UL
+#define OLDMEM_BASE		 0x10418UL
+#define OLDMEM_SIZE		 0x10420UL
+#define COMMAND_LINE		 0x10480UL
 #define COMMAND_LINE_SIZE	 896
 #define COMMAND_LINE_EXTRA       0xE000
 
@@ -29,12 +29,10 @@
 
 #define IPL_FLAG_SECURE		 0x40
 
-#define DEFAULT_IMAGE_ADDR	 0x10000
-#define DEFAULT_PSW_LOAD	 0x0008000080010000L
-#define PSW_ADDR_MASK		 0x000000007FFFFFFFL
-#define KERNEL_HEADER_SIZE	 65536
+#define DEFAULT_PSW_LOAD	 0x0008000080010000UL
+#define PSW_ADDR_MASK		 0x000000007FFFFFFFUL
 
-#define UNSPECIFIED_ADDRESS		-1ULL
+#define UNSPECIFIED_ADDRESS		-1UL
 
 
 /* IPL Parameter List header */
@@ -124,11 +122,12 @@ struct ipl_rl_hdr {
 } __packed;
 
 /* IPL Report Block header */
+/* Structure must not have any padding */
 struct ipl_rb_hdr {
 	uint32_t len;
 	uint8_t  rbt;
 	uint8_t  reserved1[11];
-} __packed;
+};
 
 /* IPL Report Block types */
 enum ipl_rbt {
@@ -162,12 +161,13 @@ struct ipl_rb_component_entry {
 #define IPL_RB_COMPONENT_FLAG_SIGNED	0x80
 #define IPL_RB_COMPONENT_FLAG_VERIFIED	0x40
 
+/* Structure must not have any padding */
 struct ipl_rb_components {
 	uint32_t len;
 	uint8_t  rbt;
 	uint8_t  reserved1[11];
 	struct ipl_rb_component_entry entries[];
-} __packed;
+};
 
 extern unsigned long long _parm_addr;   /* address of parmline */
 extern unsigned long long _initrd_addr; /* address of initrd */
