@@ -19,27 +19,29 @@
 #define ZIPL_MAGIC_SIZE			4
 #define DISK_LAYOUT_ID			0x00000001
 
-#define ZIPL_STAGE2_LOAD_ADDRESS        0x2000
-#define ZIPL_STAGE3_ENTRY_ADDRESS	0xa000LL
-#define DEFAULT_IMAGE_ADDRESS		0x10000LL
-#define KDUMP_IMAGE_ADDRESS		0x10010LL
-#define DEFAULT_STAGE3_ADDRESS          0xa000LL
-#define DEFAULT_STAGE3_PARAMS_ADDRESS	0x9000LL
-#define MINIMUM_ADDRESS 		0x10000LL
-#define ADDRESS_LIMIT 			0x80000000LL
+#define STAGE3_ENTRY			0xa000UL
+#define IMAGE_ENTRY			0x10000UL
+#define IMAGE_ENTRY_KDUMP		0x10010UL
+
+#define STAGE2_LOAD_ADDRESS		0x2000UL
+#define STAGE3_LOAD_ADDRESS		0xa000UL
+#define IMAGE_LOAD_ADDRESS		IMAGE_ENTRY
+
+#define ADDRESS_LIMIT			0x80000000UL
 #define ADDRESS_LIMIT_KDUMP		0x2000000UL /* HSA size: 32 MiB */
-#define UNSPECIFIED_ADDRESS		-1ULL
-#define MAXIMUM_PARMLINE_SIZE 		0x380
-#define MAXIMUM_PHYSICAL_BLOCKSIZE 	0x1000
+#define UNSPECIFIED_ADDRESS		-1UL
+#define MAXIMUM_PARMLINE_SIZE		0x380UL
+#define MAXIMUM_PHYSICAL_BLOCKSIZE	0x1000UL
 
-#define STAGE3_HEAP_SIZE		0x4000
-#define STAGE3_HEAP_ADDRESS		0x2000
-#define STAGE3_STACK_SIZE		0x1000
-#define STAGE3_STACK_ADDRESS		0xF000
+#define STAGE3_HEAP_SIZE		0x4000UL
+#define STAGE3_HEAP_ADDRESS		0x2000UL
+#define STAGE3_STACK_SIZE		0x1000UL
+#define STAGE3_STACK_ADDRESS		0xF000UL
+#define STAGE3_PARAMS_ADDRESS		0x9000UL
 
-#define PSW_ADDRESS_MASK		0x000000007fffffffLL
-#define PSW_LOAD                        0x0008000080000000LL
-#define PSW_DISABLED_WAIT               0x000a000000000000LL
+#define PSW_ADDRESS_MASK		0x000000007fffffffUL
+#define PSW_LOAD			0x0008000080000000UL
+#define PSW_DISABLED_WAIT		0x000a000000000000UL
 
 #define BOOTMAP_FILENAME		"bootmap"
 #define BOOTMAP_TEMPLATE_FILENAME	"bootmap_temp.XXXXXX"
@@ -47,7 +49,9 @@
 #define DEFAULTBOOT_SECTION		"defaultboot"
 
 #define ZIPL_CONF_VAR			"ZIPLCONF"
+#define ZIPL_RUNTIME_CONF		"/run/zipl/zipl.conf"
 #define ZIPL_DEFAULT_CONF		TOOLS_SYSCONFDIR "/zipl.conf"
+#define ZIPL_MINIMAL_CONF		TOOLS_LIBDIR "/zipl.conf"
 #define ZIPL_DEFAULT_BLSDIR		"/boot/loader/entries"
 #define ZIPL_STAGE3_PATH		TOOLS_LIBDIR "/stage3.bin"
 #define ZIPL_SIPL_PATH			"/sys/firmware/ipl/has_secure"
