@@ -18,6 +18,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include "lib/zt_common.h"
 #include "lib/util_part.h"
 #include "lib/util_path.h"
 
@@ -223,7 +224,7 @@ struct component_entry {
 	uint8_t data[23];
 	uint8_t type;
 	component_data compdat;
-} __attribute((packed));
+} __packed;
 
 typedef enum {
 	component_execute = 0x01,
@@ -263,7 +264,7 @@ struct component_header {
 	uint8_t magic[4];
 	uint8_t type;
 	uint8_t reserved[27];
-}  __attribute((packed));
+}  __packed;
 
 typedef enum {
 	component_header_ipl = 0x00,
@@ -626,7 +627,7 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	}
 
 	/* Add stage 3 loader to bootmap */
-	rc = add_component_file(fd, ZIPL_STAGE3_PATH, DEFAULT_STAGE3_ADDRESS,
+	rc = add_component_file(fd, ZIPL_STAGE3_PATH, STAGE3_LOAD_ADDRESS,
 				signature_size, VOID_ADD(table, offset), 1,
 				info, target, &comp_loc[comp_nr]);
 	if (rc) {
@@ -643,17 +644,17 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	rc = boot_get_stage3_parms(&stage3_params, &stage3_params_size,
 				   ipl->parm_addr, ipl->ramdisk_addr,
 				   ramdisk_size,
-				   ipl->is_kdump ? ipl->image_addr + 0x10 :
-				   ipl->image_addr,
+				   ipl->is_kdump ? IMAGE_ENTRY_KDUMP :
+				   IMAGE_ENTRY,
 				   (info->type == disk_type_scsi) ? 0 : 1,
-				   flags, image_size);
+				   flags, ipl->image_addr, image_size);
 	if (rc) {
 		free(table);
 		return rc;
 	}
 	rc = add_component_buffer(fd, stage3_params, stage3_params_size,
 				  (component_data) (uint64_t)
-				  DEFAULT_STAGE3_PARAMS_ADDRESS,
+				  STAGE3_PARAMS_ADDRESS,
 				  VOID_ADD(table, offset), info,
 				  &comp_loc[comp_nr], component_load);
 	free(stage3_params);
@@ -791,7 +792,7 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	create_component_entry(VOID_ADD(table, offset), NULL,
 			       component_execute,
 			       (component_data) (uint64_t)
-			       (ZIPL_STAGE3_ENTRY_ADDRESS | PSW_LOAD),
+			       (STAGE3_ENTRY | PSW_LOAD),
 			       info);
 	/* Write component table */
 	rc = disk_write_block_aligned(fd, table, info->phy_block_size,
@@ -1174,7 +1175,7 @@ bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
 		ulong unused_size;
 
 		/* Use approximated stage 3 size as starting point */
-		size = MINIMUM_ADDRESS;
+		size = IMAGE_LOAD_ADDRESS;
 
 		/* Ramdisk */
 		if (job->data.dump.ramdisk != NULL) {
@@ -1186,7 +1187,7 @@ bootmap_create(struct job_data *job, disk_blockptr_t *program_table,
 		/* Kernel */
 		if (stat(job->data.dump.image, &st))
 			goto out_misc_free_temp_dev;
-		size += DIV_ROUND_UP(st.st_size - 0x10000,
+		size += DIV_ROUND_UP(st.st_size - IMAGE_LOAD_ADDRESS,
 				     info->phy_block_size);
 		/* Parmfile */
 		size += DIV_ROUND_UP(DUMP_PARAM_MAX_LEN, info->phy_block_size);
