@@ -23,6 +23,7 @@
 #include "job.h"
 #include "misc.h"
 #include "scan.h"
+#include "zipl.h"
 
 /* Command line options */
 static struct option options[] = {
@@ -59,6 +60,14 @@ static struct option options[] = {
 /* Command line option abbreviations */
 static const char option_string[] = "-c:b:t:i:r:p:P:d:D:M:s:S:m:hHnVvaT:fk:";
 
+/* Locations of zipl.conf configuration file */
+static const char *zipl_conf[] = {
+	ZIPL_RUNTIME_CONF,
+	ZIPL_DEFAULT_CONF,
+	ZIPL_MINIMAL_CONF,
+	NULL
+};
+
 struct command_line {
 	char* data[SCAN_KEYWORD_NUM];
 	char* config;
@@ -647,12 +656,12 @@ check_component_address_data(struct component_loc *cl, int num, char *name,
 				     address_limit);
 			return -1;
 		}
-		if (*cl[i].addrp < MINIMUM_ADDRESS) {
+		if (*cl[i].addrp < IMAGE_LOAD_ADDRESS) {
 			if (name != NULL)
 				error_text("Section '%s'", name);
 			error_reason("Component '%s' falls below available "
 				     "address space (limit is 0x%08x)",
-				     cl[i].name, MINIMUM_ADDRESS);
+				     cl[i].name, IMAGE_LOAD_ADDRESS);
 			return -1;
 		}
 	}
@@ -690,12 +699,12 @@ finalize_component_address_data(struct component_loc *cl, int num,
 		for (j = -1; j < i; j++) {
 			if (j < 0) {
 				/* Try address before first component */
-				addr = MINIMUM_ADDRESS;
+				addr = IMAGE_LOAD_ADDRESS;
 			} else {
 				/* Try address after component j */
 				addr = *cl[j].addrp + cl[j].size;
-				if (addr < MINIMUM_ADDRESS)
-					addr = MINIMUM_ADDRESS;
+				if (addr < IMAGE_LOAD_ADDRESS)
+					addr = IMAGE_LOAD_ADDRESS;
 			}
 			addr = ALIGN(addr, cl[i].align);
 			if (addr + cl[i].size > address_limit) {
@@ -889,7 +898,7 @@ check_job_dump_images(struct job_dump_data* dump, char* name)
 	dump->image = misc_strdup(ZFCPDUMP_IMAGE);
 	if (dump->image == NULL)
 		return -1;
-	dump->image_addr = DEFAULT_IMAGE_ADDRESS;
+	dump->image_addr = IMAGE_LOAD_ADDRESS;
 
 	/* Ramdisk is no longer required with new initramfs dump system */
 	if (misc_check_readable_file(ZFCPDUMP_INITRD))
@@ -1339,7 +1348,7 @@ get_job_from_section_data(char* data[], struct job_data* job, char* section)
 			return -1;
 		if (extract_address(job->data.ipl.image,
 				    &job->data.ipl.image_addr)) {
-			job->data.ipl.image_addr = DEFAULT_IMAGE_ADDRESS;
+			job->data.ipl.image_addr = IMAGE_LOAD_ADDRESS;
 		}
 		/* Fill in parmline */
 		rc = get_parmline(data[(int) scan_keyword_parmfile],
@@ -1394,7 +1403,7 @@ get_job_from_section_data(char* data[], struct job_data* job, char* section)
 			return -1;
 		if (extract_address(job->data.ipl_tape.image,
 				    &job->data.ipl_tape.image_addr)) {
-			job->data.ipl_tape.image_addr = DEFAULT_IMAGE_ADDRESS;
+			job->data.ipl_tape.image_addr = IMAGE_LOAD_ADDRESS;
 		}
 		/* Fill in parmline */
 		rc = get_parmline(data[(int) scan_keyword_parmfile],
@@ -1780,10 +1789,10 @@ get_job_from_config_file(struct command_line* cmdline, struct job_data* job)
 {
 	struct scan_token* scan;
 	struct scan_token* new_scan;
-	char* filename;
+	const char *filename = NULL;
 	char *blsdir;
 	char* source;
-	int rc, scan_size;
+	int i, rc, scan_size;
 
 	/* Read configuration file */
 	if (cmdline->config != NULL) {
@@ -1797,7 +1806,16 @@ get_job_from_config_file(struct command_line* cmdline, struct job_data* job)
 			 ZIPL_CONF_VAR ")";
 	} else {
 		/* Use default config file */
-		filename = ZIPL_DEFAULT_CONF;
+		for (i = 0; zipl_conf[i]; i++) {
+			if (misc_check_readable_file(zipl_conf[i]) == 0) {
+				filename = zipl_conf[i];
+				break;
+			}
+		}
+		if (filename == NULL) {
+			error_text("No zipl configuration was readable");
+			return -1;
+		}
 		source = "";
 	}
 	printf("Using config file '%s'%s\n", filename, source);
@@ -1840,6 +1858,12 @@ get_job_from_config_file(struct command_line* cmdline, struct job_data* job)
 		scan_free(scan);
 		return rc;
 	}
+	rc = scan_check_bls(scan);
+	if (rc) {
+		error_text("BLS parsing '%s'", blsdir);
+		scan_free(scan);
+		return rc;
+	}
 	/* Get job from config file data */
 	if (cmdline->menu != NULL)
 		rc = get_menu_job(scan, cmdline->menu, job);
