@@ -755,8 +755,14 @@ scan_bls(const char* blsdir, struct scan_token** token, int scan_size)
 
 	remaining = scan_size - count;
 
-	if (remaining < n) {
-		size = scan_size - remaining + n;
+	/* The array of scanned tokens is allocated when the zipl config file is
+	 * parsed. Its size is a multiple of INITIAL_ARRAY_LENGTH so it may have
+	 * enough space to scan all the tokens that are defined in the BLS files.
+	 * Calculate if is enough assuming that a BLS fragment can contain up to
+	 * 4 tokens: a section heading and 3 keywords (image, ramdisk, parameter).
+	 */
+	if (remaining < n * 4) {
+		size = scan_size - remaining + (n * 4);
 		buffer = (struct scan_token *)misc_malloc(size * sizeof(struct scan_token));
 		if (!buffer)
 			goto err;
@@ -1558,6 +1564,64 @@ scan_check(struct scan_token* scan)
 	return 0;
 }
 
+/*
+ * Check if kernel and initrd image paths provided by BLS files are readable.
+ * If not, add value of 'scan_keyword_target' into search path and silently
+ * update scan list.
+ */
+int
+scan_check_bls(struct scan_token *scan)
+{
+	int i, rc;
+	char *target_value = NULL;
+	char *img_value = NULL;
+	char *buffer = NULL;
+	/*
+	 * In the BLS case, each BLS section heading inherits a keyword
+	 * assignment target= from zipl.conf, and they are all the same.
+	 *
+	 */
+	for (i = 0 ; scan[i].id != scan_id_empty; i++) {
+		if (scan[i].id == scan_id_keyword_assignment &&
+		    scan[i].content.keyword.keyword == scan_keyword_target) {
+			target_value = scan[i].content.keyword.value;
+			break;
+		}
+	}
+	if (!target_value)
+		return -1;
+	for (i = 0 ; scan[i].id != scan_id_empty; i++) {
+		if (scan[i].id != scan_id_keyword_assignment)
+			continue;
+		if (scan[i].content.keyword.keyword == scan_keyword_image ||
+		    scan[i].content.keyword.keyword == scan_keyword_ramdisk) {
+
+			rc = misc_check_readable_file(
+				scan[i].content.keyword.value);
+			if (rc) {
+				misc_asprintf(&img_value, "%s%s",
+					      target_value,
+					      scan[i].content.keyword.value);
+				rc = misc_check_readable_file(img_value);
+				if (rc) {
+					error_text(
+						"Image file '%s' is not accessible",
+						scan[i].content.keyword.value);
+					return rc;
+				}
+				buffer = (char *)
+					misc_malloc(strlen(img_value) + 1);
+				if (buffer == NULL)
+					return -1;
+				memcpy(buffer, img_value, strlen(img_value));
+				buffer[strlen(img_value)] = 0;
+				free(scan[i].content.keyword.value);
+				scan[i].content.keyword.value = buffer;
+			}
+		}
+	}
+	return 0;
+}
 
 static int
 scan_get_defaultboot_type(char* keyword[], int line[], int section_line,
