@@ -34,6 +34,7 @@
 
 #include "misc.h"
 #include "pkey.h"
+#include "cca.h"
 
 /* Detect if cryptsetup 2.1 or later is available */
 #ifdef CRYPT_LOG_DEBUG_JSON
@@ -94,6 +95,8 @@ static struct zkey_cryptsetup_globals {
 	long long keyfile_offset;
 	long long keyfile_size;
 	long long tries;
+	bool tonew;
+	bool fromold;
 	bool complete;
 	bool inplace;
 	bool staged;
@@ -101,8 +104,7 @@ static struct zkey_cryptsetup_globals {
 	bool batch_mode;
 	bool debug;
 	bool verbose;
-	void *lib_csulcca;
-	t_CSNBKTC dll_CSNBKTC;
+	struct cca_lib cca;
 	int pkey_fd;
 	struct crypt_device *cd;
 } g = {
@@ -162,6 +164,22 @@ static struct util_opt opt_vec[] = {
 		.desc = "OPTIONS",
 		.command = COMMAND_REENCIPHER,
 	},
+	{
+		.option = {"to-new", 0, NULL, 'N'},
+		.desc = "Re-enciphers a secure volume key in the LUKS2 header "
+			"that is currently enciphered with the master key in "
+			"the CURRENT register with the master key in the NEW "
+			"register",
+		.command = COMMAND_REENCIPHER,
+	},
+	{
+		.option = {"from-old", 0, NULL, 'O'},
+		.desc = "Re-enciphers a secure volume key in the LUKS2 header "
+			"that is currently enciphered with the master key in "
+			"the OLD register with the master key in the CURRENT "
+			"register",
+		.command = COMMAND_REENCIPHER,
+	},
 	{
 		.option = {"staged", 0, NULL, 's'},
 		.desc = "Forces that the re-enciphering of a secure volume "
@@ -1311,22 +1329,25 @@ static int activate_unbound_keyslot(int token, int keyslot, const char *key,
 	return rc;
 }
 
-static int check_keysize_and_cipher_mode(size_t keysize)
+static int check_keysize_and_cipher_mode(const u8 *key, size_t keysize)
 {
-	if (keysize == 0) {
+	if (keysize < MIN_SECURE_KEY_SIZE ||
+	    keysize > 2 * MAX_SECURE_KEY_SIZE) {
 		warnx("Invalid volume key size");
 		return -EINVAL;
 	}
 
 	if (strncmp(crypt_get_cipher_mode(g.cd), "xts", 3) == 0) {
-		if (keysize != 2 * SECURE_KEY_SIZE) {
+		if (keysize < 2 * MIN_SECURE_KEY_SIZE ||
+		    (key != NULL && !is_xts_key(key, keysize))) {
 			warnx("The volume key size %lu is not valid for the "
 			      "cipher mode '%s'", keysize,
 			      crypt_get_cipher_mode(g.cd));
 			return -EINVAL;
 		}
 	} else {
-		if (keysize != SECURE_KEY_SIZE) {
+		if (keysize > MAX_SECURE_KEY_SIZE ||
+		    (key != NULL && is_xts_key(key, keysize))) {
 			warnx("The volume key size %lu is not valid for the "
 			      "cipher mode '%s'", keysize,
 			      crypt_get_cipher_mode(g.cd));
@@ -1359,7 +1380,7 @@ static int open_keyslot(int keyslot, char **key, size_t *keysize,
 	vkeysize = crypt_get_volume_key_size(g.cd);
 	pr_verbose("Volume key size: %lu", vkeysize);
 
-	rc = check_keysize_and_cipher_mode(vkeysize);
+	rc = check_keysize_and_cipher_mode(NULL, vkeysize);
 	if (rc != 0)
 		return rc;
 
@@ -1471,7 +1492,7 @@ static int validate_keyslot(int keyslot, char **key, size_t *keysize,
 	keyslot = rc;
 
 	rc = validate_secure_key(g.pkey_fd, (u8 *)vkey, vkeysize, clear_keysize,
-				 &is_old, g.verbose);
+				 &is_old, NULL, g.verbose);
 	if (rc != 0) {
 		if (invalid_msg != NULL)
 			warnx("%s", invalid_msg);
@@ -1514,10 +1535,12 @@ static int reencipher_prepare(int token)
 	char *password = NULL;
 	size_t password_len;
 	char *key = NULL;
+	int selected = 1;
 	size_t keysize;
 	int is_old_mk;
 	char *prompt;
 	char *msg;
+	u64 mkvp;
 	int rc;
 
 	if (token >= 0) {
@@ -1551,7 +1574,7 @@ static int reencipher_prepare(int token)
 	if (rc != 0)
 		goto out;
 
-	rc = generate_key_verification_pattern(key, keysize,
+	rc = generate_key_verification_pattern((u8 *)key, keysize,
 					       reenc_tok.verification_pattern,
 					 sizeof(reenc_tok.verification_pattern),
 					       g.verbose);
@@ -1570,25 +1593,97 @@ static int reencipher_prepare(int token)
 	if (rc < 0)
 		goto out;
 
-	util_asprintf(&msg, "The secure volume key of device '%s' is "
-		      "enciphered with the %s CCA master key and is being "
-		      "re-enciphered with the %s CCA master key.",
-		      g.pos_arg, is_old_mk ? "OLD" : "CURRENT",
-		      is_old_mk ? "CURRENT" : "NEW");
-	util_print_indented(msg, 0);
-	free(msg);
+	if (!g.fromold && !g.tonew) {
+		/* Autodetect reencipher mode */
+		if (is_old_mk) {
+			g.fromold = 1;
+			util_asprintf(&msg, "The secure volume key of device "
+				      "'%s' is enciphered with the OLD CCA "
+				      "master key and is being re-enciphered "
+				      "with the CURRENT CCA master key.",
+				      g.pos_arg);
+			util_print_indented(msg, 0);
+			free(msg);
+		} else {
+			g.tonew = 1;
+			util_asprintf(&msg, "The secure volume key of device "
+				      "'%s' is enciphered with the CURRENT CCA "
+				      "master key and is being re-enciphered "
+				      "with the NEW CCA master key.",
+				      g.pos_arg);
+			util_print_indented(msg, 0);
+			free(msg);
+		}
+	}
 
-	rc = key_token_change(g.dll_CSNBKTC, (u8 *)key, keysize,
-			      is_old_mk ? METHOD_OLD_TO_CURRENT :
-					  METHOD_CURRENT_TO_NEW,
-			      g.verbose);
+	rc = get_master_key_verification_pattern((u8 *)key, keysize, &mkvp,
+						 g.verbose);
 	if (rc != 0) {
-		warnx("Failed to re-encipher the secure volume key of device "
-		      "'%s'", g.pos_arg);
-		rc = -EINVAL;
+		warnx("Failed to get the master key verification pattern: %s",
+		      strerror(-rc));
 		goto out;
 	}
 
+	if (g.fromold) {
+		rc = select_cca_adapter_by_mkvp(&g.cca, mkvp, NULL,
+						FLAG_SEL_CCA_MATCH_OLD_MKVP,
+						g.verbose);
+		if (rc == -ENOTSUP) {
+			rc = 0;
+			selected = 0;
+		}
+		if (rc != 0) {
+			util_print_indented("No APQN found that is suitable "
+					    "for re-enciphering the secure AES "
+					    "volume key from the OLD to the "
+					    "CURRENT CCA master key.", 0);
+			goto out;
+		}
+
+		rc = key_token_change(&g.cca, (u8 *)key, keysize,
+				      METHOD_OLD_TO_CURRENT, g.verbose);
+		if (rc != 0) {
+			warnx("Failed to re-encipher the secure volume key of "
+			      "device '%s'\n", g.pos_arg);
+			if (!selected)
+				print_msg_for_cca_envvars(
+						"secure AES volume key");
+			rc = -EINVAL;
+			goto out;
+		}
+	}
+
+	if (g.tonew) {
+		rc = select_cca_adapter_by_mkvp(&g.cca, mkvp, NULL,
+						FLAG_SEL_CCA_MATCH_CUR_MKVP |
+						FLAG_SEL_CCA_NEW_MUST_BE_SET,
+						g.verbose);
+		if (rc == -ENOTSUP) {
+			rc = 0;
+			selected = 0;
+		}
+		if (rc != 0) {
+			util_print_indented("No APQN found that is suitable "
+					    "for re-enciphering the secure AES "
+					    "volume key from the CURRENT to "
+					    "the NEW CCA master key.", 0);
+			goto out;
+		}
+
+		rc = key_token_change(&g.cca, (u8 *)key, keysize,
+				      METHOD_CURRENT_TO_NEW,
+				      g.verbose);
+		if (rc != 0) {
+			warnx("Failed to re-encipher the secure volume key of "
+			      "device '%s'\n", g.pos_arg);
+			if (!selected)
+				print_msg_for_cca_envvars(
+						"secure AES volume key");
+			rc = -EINVAL;
+			goto out;
+		}
+	}
+
 	rc = crypt_keyslot_add_by_key(g.cd, CRYPT_ANY_SLOT, key, keysize,
 				      password, password_len,
 				      CRYPT_VOLUME_KEY_NO_SEGMENT);
@@ -1651,10 +1746,12 @@ static int reencipher_complete(int token)
 	char *password = NULL;
 	size_t password_len;
 	char *key = NULL;
+	int selected = 1;
 	size_t keysize;
 	int is_old_mk;
 	char *prompt;
 	char *msg;
+	u64 mkvp;
 	int rc;
 
 	rc = get_reencipher_token(g.cd, token, &tok, true);
@@ -1700,11 +1797,38 @@ static int reencipher_complete(int token)
 			goto out;
 		}
 
-		rc = key_token_change(g.dll_CSNBKTC, (u8 *)key, keysize,
+		rc = get_master_key_verification_pattern((u8 *)key, keysize,
+							 &mkvp, g.verbose);
+		if (rc != 0) {
+			warnx("Failed to get the master key verification "
+			      "pattern: %s",
+			      strerror(-rc));
+			goto out;
+		}
+
+		rc = select_cca_adapter_by_mkvp(&g.cca, mkvp, NULL,
+						FLAG_SEL_CCA_MATCH_OLD_MKVP,
+						g.verbose);
+		if (rc == -ENOTSUP) {
+			rc = 0;
+			selected = 0;
+		}
+		if (rc != 0) {
+			util_print_indented("No APQN found that is suitable "
+					    "for re-enciphering the secure AES "
+					    "volume key from the OLD to the "
+					    "CURRENT CCA master key.", 0);
+			goto out;
+		}
+
+		rc = key_token_change(&g.cca, (u8 *)key, keysize,
 				      METHOD_OLD_TO_CURRENT, g.verbose);
 		if (rc != 0) {
 			warnx("Failed to re-encipher the secure volume key for "
-			      "device '%s'", g.pos_arg);
+			      "device '%s'\n", g.pos_arg);
+			if (!selected)
+				print_msg_for_cca_envvars(
+						"secure AES volume key");
 			rc = -EINVAL;
 			goto out;
 		}
@@ -1730,8 +1854,8 @@ static int reencipher_complete(int token)
 
 	}
 
-	rc = generate_key_verification_pattern(key, keysize, vp, sizeof(vp),
-					       g.verbose);
+	rc = generate_key_verification_pattern((u8 *)key, keysize, vp,
+					       sizeof(vp), g.verbose);
 	if (rc != 0) {
 		warnx("Failed to generate the verification pattern: %s",
 		      strerror(-rc));
@@ -1834,6 +1958,7 @@ static int command_validate(void)
 	char *prompt;
 	char *msg;
 	int token;
+	u64 mkvp;
 	int rc;
 
 	util_asprintf(&prompt, "Enter passphrase for '%s': ", g.pos_arg);
@@ -1847,7 +1972,7 @@ static int command_validate(void)
 		goto out;
 
 	rc = validate_secure_key(g.pkey_fd, (u8 *)key, keysize, &clear_keysize,
-				 &is_old_mk, g.verbose);
+				 &is_old_mk, NULL, g.verbose);
 	is_valid = (rc == 0);
 
 	token = find_token(g.cd, PAES_REENC_TOKEN_NAME);
@@ -1864,18 +1989,29 @@ static int command_validate(void)
 			vp_tok_avail = 1;
 	}
 
+	rc = get_master_key_verification_pattern((u8 *)key, keysize,
+						 &mkvp, g.verbose);
+	if (rc != 0) {
+		warnx("Failed to get the master key verification pattern: %s",
+		      strerror(-rc));
+		goto out;
+	}
+
 	printf("Validation of secure volume key of device '%s':\n", g.pos_arg);
 	printf("  Status:                %s\n", is_valid ? "Valid" : "Invalid");
 	printf("  Secure key size:       %lu bytes\n", keysize);
 	printf("  XTS type key:          %s\n",
-	       keysize > SECURE_KEY_SIZE ? "Yes" : "No");
+	       is_xts_key((u8 *)key, keysize) ? "Yes" : "No");
+	printf("  Key type:              %s\n",
+	       get_key_type((u8 *)key, keysize));
 	if (is_valid) {
 		printf("  Clear key size:        %lu bits\n", clear_keysize);
-		printf("  Enciphered with:       %s CCA master key\n",
-		       is_old_mk ? "OLD" : "CURRENT");
+		printf("  Enciphered with:       %s CCA master key (MKVP: "
+		       "%016llx)\n", is_old_mk ? "OLD" : "CURRENT", mkvp);
 	} else {
 		printf("  Clear key size:        (unknown)\n");
-		printf("  Enciphered with:       (unknown)\n");
+		printf("  Enciphered with:       (unknown, MKVP: %016llx)\n",
+		       mkvp);
 	}
 	if (vp_tok_avail)
 		print_verification_pattern(vp_tok.verification_pattern);
@@ -1943,7 +2079,7 @@ static int command_setvp(void)
 
 	token = find_token(g.cd, PAES_VP_TOKEN_NAME);
 
-	rc = generate_key_verification_pattern(key, keysize,
+	rc = generate_key_verification_pattern((const u8 *)key, keysize,
 					       vp_tok.verification_pattern,
 					    sizeof(vp_tok.verification_pattern),
 					       g.verbose);
@@ -1998,12 +2134,12 @@ static int command_setkey(void)
 	if (newkey == NULL)
 		return EXIT_FAILURE;
 
-	rc = check_keysize_and_cipher_mode(newkey_size);
+	rc = check_keysize_and_cipher_mode(newkey, newkey_size);
 	if (rc != 0)
 		goto out;
 
 	rc = validate_secure_key(g.pkey_fd, newkey, newkey_size, NULL,
-				 &is_old_mk, g.verbose);
+				 &is_old_mk, NULL, g.verbose);
 	if (rc != 0) {
 		warnx("The secure key in file '%s' is not valid",
 		      g.master_key_file);
@@ -2033,21 +2169,14 @@ static int command_setkey(void)
 	if (rc < 0)
 		goto out;
 
-	if (keysize != newkey_size) {
-		warnx("The secure key in file '%s' has an invalid size",
-		      g.master_key_file);
-		rc = -EINVAL;
-		goto out;
-	}
-
-	if (memcmp(newkey, key, keysize) == 0) {
+	if (keysize == newkey_size && memcmp(newkey, key, keysize) == 0) {
 		warnx("The secure key in file '%s' is equal to the current "
 		      "volume key, setkey is ignored", g.master_key_file);
 		rc = 0;
 		goto out;
 	}
 
-	rc = generate_key_verification_pattern((char *)newkey, newkey_size, vp,
+	rc = generate_key_verification_pattern(newkey, newkey_size, vp,
 					       sizeof(vp), g.verbose);
 	if (rc != 0) {
 		warnx("Failed to generate the verification pattern: %s",
@@ -2206,6 +2335,12 @@ int main(int argc, char *argv[])
 		if (c == -1)
 			break;
 		switch (c) {
+		case 'N':
+			g.tonew = 1;
+			break;
+		case 'O':
+			g.fromold = 1;
+			break;
 		case 'c':
 			g.complete = 1;
 			break;
@@ -2288,8 +2423,7 @@ int main(int argc, char *argv[])
 	}
 
 	if (command->need_cca_library) {
-		rc = load_cca_library(&g.lib_csulcca, &g.dll_CSNBKTC,
-				      g.verbose);
+		rc = load_cca_library(&g.cca, g.verbose);
 		if (rc != 0) {
 			rc = EXIT_FAILURE;
 			goto out;
@@ -2331,8 +2465,8 @@ int main(int argc, char *argv[])
 	rc = command->function();
 
 out:
-	if (g.lib_csulcca)
-		dlclose(g.lib_csulcca);
+	if (g.cca.lib_csulcca)
+		dlclose(g.cca.lib_csulcca);
 	if (g.pkey_fd >= 0)
 		close(g.pkey_fd);
 	if (g.cd)
