@@ -27,9 +27,11 @@
 #include "lib/util_prg.h"
 #include "lib/zt_common.h"
 
+#include "cca.h"
 #include "keystore.h"
 #include "misc.h"
 #include "pkey.h"
+#include "utils.h"
 
 /*
  * Program configuration
@@ -71,6 +73,7 @@ static struct zkey_globals {
 	long int sector_size;
 	char *volume_type;
 	char *newname;
+	char *key_type;
 	bool run;
 	bool batch_mode;
 	char *keyfile;
@@ -80,8 +83,7 @@ static struct zkey_globals {
 	bool force;
 	bool open;
 	bool format;
-	void *lib_csulcca;
-	t_CSNBKTC dll_CSNBKTC;
+	struct cca_lib cca;
 	int pkey_fd;
 	struct keystore *keystore;
 } g = {
@@ -104,6 +106,7 @@ static struct zkey_globals {
 #define COMMAND_COPY		"copy  "
 #define COMMAND_CRYPTTAB	"crypttab"
 #define COMMAND_CRYPTSETUP	"cryptsetup"
+#define COMMAND_CONVERT		"convert"
 
 #define ZKEY_COMMAND_MAX_LEN	10
 
@@ -161,7 +164,7 @@ static struct util_opt opt_vec[] = {
 		.desc = "Name of the secure AES key in the repository. If "
 			"option --name/-N is specified, then the generated "
 			"secure AES key is stored in the repository. Parameter "
-			"SECURE-KEY-FILE is not used when option --name/-M is "
+			"SECURE-KEY-FILE is not used when option --name/-N is "
 			"specified",
 		.command = COMMAND_GENERATE,
 	},
@@ -215,6 +218,15 @@ static struct util_opt opt_vec[] = {
 		.command = COMMAND_GENERATE,
 	},
 #endif
+	{
+		.option = { "key-type", required_argument, NULL, 'K'},
+		.argument = "type",
+		.desc = "The type of the key. Possible values are '"
+			KEY_TYPE_CCA_AESDATA"' and '"KEY_TYPE_CCA_AESCIPHER"'. "
+			"When this option is omitted, the default is '"
+			KEY_TYPE_CCA_AESDATA"'",
+		.command = COMMAND_GENERATE,
+	},
 	/***********************************************************/
 	{
 		.flags = UTIL_OPT_FLAG_SECTION,
@@ -431,6 +443,15 @@ static struct util_opt opt_vec[] = {
 		.command = COMMAND_LIST,
 	},
 #endif
+	{
+		.option = { "key-type", required_argument, NULL, 'K'},
+		.argument = "type",
+		.desc = "The type of the key. Possible values are '"
+			KEY_TYPE_CCA_AESDATA"' and '"KEY_TYPE_CCA_AESCIPHER"'. "
+			"Use this option to list all keys with the specified "
+			"key type.",
+		.command = COMMAND_LIST,
+	},
 	/***********************************************************/
 	{
 		.flags = UTIL_OPT_FLAG_SECTION,
@@ -744,6 +765,38 @@ static struct util_opt opt_vec[] = {
 		.flags = UTIL_OPT_FLAG_NOSHORT,
 	},
 #endif
+	/***********************************************************/
+	{
+		.flags = UTIL_OPT_FLAG_SECTION,
+		.desc = "OPTIONS",
+		.command = COMMAND_CONVERT,
+	},
+	{
+		.option = { "name", required_argument, NULL, 'N'},
+		.argument = "NAME",
+		.desc = "Name of the secure AES key in the repository that is "
+			"to be converted",
+		.command = COMMAND_CONVERT,
+	},
+	{
+		.option = { "key-type", required_argument, NULL, 'K'},
+		.argument = "type",
+		.desc = "The type of the key to convert the secure key to. "
+			"Possible values are '"KEY_TYPE_CCA_AESCIPHER"'. ",
+		.command = COMMAND_CONVERT,
+	},
+	{
+		.option = {"no-apqn-check", 0, NULL, OPT_NO_APQN_CHECK},
+		.desc = "Do not check if the associated APQN(s) are available",
+		.command = COMMAND_CONVERT,
+		.flags = UTIL_OPT_FLAG_NOSHORT,
+	},
+	{
+		.option = {"force", 0, NULL, 'F'},
+		.desc = "Do not prompt for a confirmation when converting a "
+			"key",
+		.command = COMMAND_CONVERT,
+	},
 	/***********************************************************/
 	{
 		.flags = UTIL_OPT_FLAG_SECTION,
@@ -792,6 +845,7 @@ static int command_rename(void);
 static int command_copy(void);
 static int command_crypttab(void);
 static int command_cryptsetup(void);
+static int command_convert(void);
 
 static struct zkey_command zkey_commands[] = {
 	{
@@ -926,6 +980,21 @@ static struct zkey_command zkey_commands[] = {
 		.has_options = 1,
 		.need_keystore = 1,
 	},
+	{
+		.command = COMMAND_CONVERT,
+		.abbrev_len = 3,
+		.function = command_convert,
+		.need_cca_library = 1,
+		.need_pkey_device = 1,
+		.short_desc = "Convert a secure AES key",
+		.long_desc = "Convert an existing secure AES key that is "
+			     "either contained in SECURE-KEY-FILE or is stored "
+			     "in the repository from one key type to another "
+			     "type.",
+		.has_options = 1,
+		.pos_arg = "[SECURE-KEY-FILE]",
+		.pos_arg_optional = 1,
+	},
 	{ .command = NULL }
 };
 
@@ -1008,9 +1077,8 @@ static int command_generate_clear(void)
 
 	rc = generate_secure_key_clear(g.pkey_fd, g.pos_arg,
 				       g.keybits, g.xts,
-				       g.clearkeyfile,
-				       AUTOSELECT, AUTOSELECT,
-				       g.verbose);
+				       g.clearkeyfile, g.key_type,
+				       NULL, g.verbose);
 
 	return rc != 0 ? EXIT_FAILURE : EXIT_SUCCESS;
 }
@@ -1025,9 +1093,8 @@ static int command_generate_random(void)
 	int rc;
 
 	rc = generate_secure_key_random(g.pkey_fd, g.pos_arg,
-					g.keybits, g.xts,
-					AUTOSELECT, AUTOSELECT,
-					g.verbose);
+					g.keybits, g.xts, g.key_type,
+					NULL, g.verbose);
 
 	return rc != 0 ? EXIT_FAILURE : EXIT_SUCCESS;
 }
@@ -1047,7 +1114,7 @@ static int command_generate_repository(void)
 	rc = keystore_generate_key(g.keystore, g.name, g.description, g.volumes,
 				   g.apqns, g.noapqncheck, g.sector_size,
 				   g.keybits, g.xts, g.clearkeyfile,
-				   g.volume_type, g.pkey_fd);
+				   g.volume_type, g.key_type, g.pkey_fd);
 
 	return rc != 0 ? EXIT_FAILURE : EXIT_SUCCESS;
 }
@@ -1060,6 +1127,8 @@ static int command_generate_repository(void)
  */
 static int command_generate(void)
 {
+	int rc;
+
 	if (g.pos_arg != NULL && g.name != NULL) {
 		warnx(" Option '--name|-N' is not valid for generating a key "
 		      "outside of the repository");
@@ -1072,6 +1141,8 @@ static int command_generate(void)
 		util_prg_print_parse_error();
 		return EXIT_FAILURE;
 	}
+	if (g.key_type == NULL)
+		g.key_type = KEY_TYPE_CCA_AESDATA;
 	if (g.name != NULL)
 		return command_generate_repository();
 	if (g.pos_arg != NULL) {
@@ -1100,6 +1171,16 @@ static int command_generate(void)
 			return EXIT_FAILURE;
 		}
 
+		rc = cross_check_apqns(NULL, 0,
+				get_min_card_level_for_keytype(g.key_type),
+				true, g.verbose);
+		if (rc == -EINVAL)
+			return EXIT_FAILURE;
+		if (rc != 0 && rc != -ENOTSUP) {
+			warnx("Your master key setup is improper");
+			return EXIT_FAILURE;
+		}
+
 		return g.clearkeyfile ? command_generate_clear()
 				      : command_generate_random();
 	}
@@ -1117,7 +1198,9 @@ static int command_reencipher_file(void)
 {
 	size_t secure_key_size;
 	int rc, is_old_mk;
+	int selected = 1;
 	u8 *secure_key;
+	u64 mkvp;
 
 	if (g.name != NULL) {
 		warnx("Option '--name|-N' is not valid for "
@@ -1156,13 +1239,22 @@ static int command_reencipher_file(void)
 		return EXIT_FAILURE;
 
 	rc = validate_secure_key(g.pkey_fd, secure_key, secure_key_size, NULL,
-				 &is_old_mk, g.verbose);
+				 &is_old_mk, NULL, g.verbose);
 	if (rc != 0) {
 		warnx("The secure key in file '%s' is not valid", g.pos_arg);
 		rc = EXIT_FAILURE;
 		goto out;
 	}
 
+	rc = get_master_key_verification_pattern(secure_key, secure_key_size,
+						 &mkvp, g.verbose);
+	if (rc != 0) {
+		warnx("Failed to get the master key verification pattern: %s",
+		      strerror(-rc));
+		rc = EXIT_FAILURE;
+		goto out;
+	}
+
 	if (!g.fromold && !g.tonew) {
 		/* Autodetect reencipher option */
 		if (is_old_mk) {
@@ -1194,13 +1286,28 @@ static int command_reencipher_file(void)
 		pr_verbose("Secure key will be re-enciphered from OLD to the "
 			   "CURRENT CCA master key");
 
-		rc = key_token_change(g.dll_CSNBKTC,
-				      secure_key, secure_key_size,
+		rc = select_cca_adapter_by_mkvp(&g.cca, mkvp, NULL,
+						FLAG_SEL_CCA_MATCH_OLD_MKVP,
+						g.verbose);
+		if (rc == -ENOTSUP) {
+			rc = 0;
+			selected = 0;
+		}
+		if (rc != 0) {
+			warnx("No APQN found that is suitable for "
+			      "re-enciphering the secure AES volume key");
+			rc = EXIT_FAILURE;
+			goto out;
+		}
+
+		rc = key_token_change(&g.cca, secure_key, secure_key_size,
 				      METHOD_OLD_TO_CURRENT,
 				      g.verbose);
 		if (rc != 0) {
 			warnx("Re-encipher from OLD to CURRENT CCA "
-			      "master key has failed");
+			      "master key has failed\n");
+			if (!selected)
+				print_msg_for_cca_envvars("secure AES key");
 			rc = EXIT_FAILURE;
 			goto out;
 		}
@@ -1209,12 +1316,30 @@ static int command_reencipher_file(void)
 		pr_verbose("Secure key will be re-enciphered from CURRENT "
 			   "to the NEW CCA master key");
 
-		rc = key_token_change(g.dll_CSNBKTC,
-				      secure_key, secure_key_size,
+		rc = select_cca_adapter_by_mkvp(&g.cca, mkvp, NULL,
+						FLAG_SEL_CCA_MATCH_CUR_MKVP |
+						FLAG_SEL_CCA_NEW_MUST_BE_SET,
+						g.verbose);
+		if (rc == -ENOTSUP) {
+			rc = 0;
+			selected = 0;
+		}
+		if (rc != 0) {
+			util_print_indented("No APQN found that is suitable "
+					    "for re-enciphering this secure "
+					    "AES key and has the NEW master "
+					    "key loaded", 0);
+			rc = EXIT_FAILURE;
+			goto out;
+		}
+
+		rc = key_token_change(&g.cca, secure_key, secure_key_size,
 				      METHOD_CURRENT_TO_NEW, g.verbose);
 		if (rc != 0) {
 			warnx("Re-encipher from CURRENT to NEW CCA "
-			      "master key has failed");
+			      "master key has failed\n");
+			if (!selected)
+				print_msg_for_cca_envvars("secure AES key");
 			rc = EXIT_FAILURE;
 			goto out;
 		}
@@ -1270,7 +1395,7 @@ static int command_reencipher_repository(void)
 
 	rc = keystore_reencipher_key(g.keystore, g.name, g.apqns, g.fromold,
 				     g.tonew, g.inplace, g.staged, g.complete,
-				     g.pkey_fd, g.dll_CSNBKTC);
+				     g.pkey_fd, &g.cca);
 
 	return rc != 0 ? EXIT_FAILURE : EXIT_SUCCESS;
 }
@@ -1302,6 +1427,7 @@ static int command_validate_file(void)
 	size_t clear_key_size;
 	u8 *secure_key;
 	int is_old_mk;
+	u64 mkvp;
 	int rc;
 
 	if (g.name != NULL) {
@@ -1329,14 +1455,14 @@ static int command_validate_file(void)
 		return EXIT_FAILURE;
 
 	rc = validate_secure_key(g.pkey_fd, secure_key, secure_key_size,
-				 &clear_key_size, &is_old_mk, g.verbose);
+				 &clear_key_size, &is_old_mk, NULL, g.verbose);
 	if (rc != 0) {
 		warnx("The secure key in file '%s' is not valid", g.pos_arg);
 		rc = EXIT_FAILURE;
 		goto out;
 	}
 
-	rc = generate_key_verification_pattern((char *)secure_key,
+	rc = generate_key_verification_pattern(secure_key,
 					       secure_key_size, vp, sizeof(vp),
 					       g.verbose);
 	if (rc != 0) {
@@ -1348,19 +1474,42 @@ static int command_validate_file(void)
 		goto out;
 	}
 
+	rc = get_master_key_verification_pattern(secure_key, secure_key_size,
+						 &mkvp, g.verbose);
+	if (rc != 0) {
+		warnx("Failed to get the master key verification pattern: %s",
+		      strerror(-rc));
+		rc = EXIT_FAILURE;
+		goto out;
+	}
+
 	printf("Validation of secure key in file '%s':\n", g.pos_arg);
 	printf("  Status:                Valid\n");
 	printf("  Secure key size:       %lu bytes\n", secure_key_size);
+	printf("  Key type:              %s\n",
+	       get_key_type(secure_key, secure_key_size));
 	printf("  Clear key size:        %lu bits\n", clear_key_size);
 	printf("  XTS type key:          %s\n",
-	       secure_key_size > SECURE_KEY_SIZE ? "Yes" : "No");
-	printf("  Enciphered with:       %s CCA master key\n",
-	       is_old_mk ? "OLD" : "CURRENT");
+	       is_xts_key(secure_key, secure_key_size) ? "Yes" : "No");
+	printf("  Enciphered with:       %s CCA master key (MKVP: %016llx)\n",
+	       is_old_mk ? "OLD" : "CURRENT", mkvp);
 	printf("  Verification pattern:  %.*s\n", VERIFICATION_PATTERN_LEN / 2,
 	       vp);
 	printf("                         %.*s\n", VERIFICATION_PATTERN_LEN / 2,
 	       &vp[VERIFICATION_PATTERN_LEN / 2]);
 
+	rc = cross_check_apqns(NULL, mkvp,
+			       get_min_card_level_for_keytype(
+				     get_key_type(secure_key, secure_key_size)),
+			       true, g.verbose);
+	if (rc == -EINVAL)
+		return EXIT_FAILURE;
+	if (rc != 0 && rc != -ENOTSUP) {
+		warnx("Your master key setup is improper");
+		rc = EXIT_FAILURE;
+		goto out;
+	}
+
 out:
 	free(secure_key);
 	return rc;
@@ -1422,7 +1571,7 @@ static int command_import(void)
 
 	rc = keystore_import_key(g.keystore, g.name, g.description, g.volumes,
 				 g.apqns, g.noapqncheck, g.sector_size,
-				 g.pos_arg, g.volume_type);
+				 g.pos_arg, g.volume_type, &g.cca);
 
 	return rc != 0 ? EXIT_FAILURE : EXIT_SUCCESS;
 }
@@ -1456,7 +1605,7 @@ static int command_list(void)
 	int rc;
 
 	rc = keystore_list_keys(g.keystore, g.name, g.volumes, g.apqns,
-				g.volume_type);
+				g.volume_type, g.key_type);
 
 	return rc != 0 ? EXIT_FAILURE : EXIT_SUCCESS;
 }
@@ -1591,6 +1740,206 @@ static int command_cryptsetup(void)
 	return rc != 0 ? EXIT_FAILURE : EXIT_SUCCESS;
 }
 
+/*
+ * Command handler for 'convert'.
+ *
+ * Converts secure keys from one key type to another
+ */
+static int command_convert_file(void)
+{
+	u8 output_key[2 * MAX_SECURE_KEY_SIZE];
+	unsigned int output_key_size;
+	size_t secure_key_size;
+	int rc, is_old_mk;
+	int selected = 1;
+	u8 *secure_key;
+	int min_level;
+	u64 mkvp;
+
+	if (g.name != NULL) {
+		warnx("Option '--name|-N' is not valid for "
+		      "re-enciphering a key outside of the repository");
+		util_prg_print_parse_error();
+		return EXIT_FAILURE;
+	}
+	if (g.noapqncheck) {
+		warnx("Option '--no-apqn-check' is not valid for "
+		      "converting a key outside of the repository");
+		util_prg_print_parse_error();
+		return EXIT_FAILURE;
+	}
+
+	min_level = get_min_card_level_for_keytype(g.key_type);
+	if (min_level < 0) {
+		warnx("Invalid key-type specified: %s", g.key_type);
+		return EXIT_FAILURE;
+	}
+
+	rc = cross_check_apqns(NULL, 0, min_level, true, g.verbose);
+	if (rc == -EINVAL)
+		return EXIT_FAILURE;
+	if (rc != 0 && rc != -ENOTSUP) {
+		warnx("Your master key setup is improper");
+		return EXIT_FAILURE;
+	}
+
+	/* Read the secure key to be re-enciphered */
+	secure_key = read_secure_key(g.pos_arg, &secure_key_size, g.verbose);
+	if (secure_key == NULL)
+		return EXIT_FAILURE;
+
+	rc = validate_secure_key(g.pkey_fd, secure_key, secure_key_size, NULL,
+				 &is_old_mk, NULL, g.verbose);
+	if (rc != 0) {
+		warnx("The secure key in file '%s' is not valid", g.pos_arg);
+		rc = EXIT_FAILURE;
+		goto out;
+	}
+
+	rc = get_master_key_verification_pattern(secure_key, secure_key_size,
+						 &mkvp, g.verbose);
+	if (rc != 0) {
+		warnx("Failed to get the master key verification pattern: %s",
+		      strerror(-rc));
+		rc = EXIT_FAILURE;
+		goto out;
+	}
+
+	if (strcasecmp(get_key_type(secure_key, secure_key_size),
+		       g.key_type) == 0) {
+		warnx("The secure key in file '%s' is already of type %s",
+		      g.pos_arg, get_key_type(secure_key, secure_key_size));
+		rc = EXIT_FAILURE;
+		goto out;
+	}
+
+	if (is_cca_aes_data_key(secure_key, secure_key_size)) {
+		if (strcasecmp(g.key_type, KEY_TYPE_CCA_AESCIPHER) != 0) {
+			warnx("The secure key in file '%s' can not be "
+			      "converted into type %s", g.pos_arg, g.key_type);
+			rc = EXIT_FAILURE;
+			goto out;
+		}
+	} else if (is_cca_aes_cipher_key(secure_key, secure_key_size)) {
+		warnx("The secure key in file '%s' is already of type %s",
+		      g.pos_arg, KEY_TYPE_CCA_AESCIPHER);
+		rc = EXIT_FAILURE;
+		goto out;
+	} else {
+		warnx("The secure key in file '%s' has an unsupported key type",
+		      g.pos_arg);
+		rc = EXIT_FAILURE;
+		goto out;
+	}
+
+	rc = select_cca_adapter_by_mkvp(&g.cca, mkvp, NULL,
+					FLAG_SEL_CCA_MATCH_CUR_MKVP,
+					g.verbose);
+	if (rc == -ENOTSUP) {
+		rc = 0;
+		selected = 0;
+	}
+	if (rc != 0) {
+		warnx("No APQN found that is suitable for "
+		      "converting the secure AES key in file '%s'", g.pos_arg);
+		rc = EXIT_FAILURE;
+		goto out;
+	}
+
+	if (!g.force) {
+		util_print_indented("ATTENTION: Converting a secure key is "
+				    "irreversible, and might have an effect "
+				    "on the volumes encrypted with it!", 0);
+		printf("%s: Convert key in file '%s' [y/N]? ",
+		       program_invocation_short_name, g.pos_arg);
+		if (!prompt_for_yes(g.verbose)) {
+			warnx("Operation aborted");
+			rc = EXIT_FAILURE;
+			goto out;
+		}
+	}
+
+	memset(output_key, 0, sizeof(output_key));
+	output_key_size = sizeof(output_key);
+	rc = convert_aes_data_to_cipher_key(&g.cca, secure_key, secure_key_size,
+					    output_key, &output_key_size,
+					    g.verbose);
+	if (rc != 0) {
+		warnx("Converting the secure key from %s to %s has failed",
+		      get_key_type(secure_key, secure_key_size), g.key_type);
+		if (!selected)
+			print_msg_for_cca_envvars("secure AES key");
+		rc = EXIT_FAILURE;
+		goto out;
+	}
+
+	rc = restrict_key_export(&g.cca, output_key, output_key_size,
+				 g.verbose);
+	if (rc != 0) {
+		warnx("Export restricting the converted secure key has failed");
+		if (!selected)
+			print_msg_for_cca_envvars("secure AES key");
+		rc = EXIT_FAILURE;
+		goto out;
+	}
+
+	pr_verbose("Secure key was converted successfully");
+
+	/* Write the converted secure key */
+	rc = write_secure_key(g.outputfile ? g.outputfile : g.pos_arg,
+			      output_key, output_key_size, g.verbose);
+	if (rc != 0)
+		rc = EXIT_FAILURE;
+out:
+	free(secure_key);
+	return rc;
+}
+
+/*
+ * Command handler for 'convert in repository'.
+ *
+ * Converts secure keys from one key type to another
+ */
+static int command_convert_repository(void)
+{
+	int rc;
+
+	if (g.name == NULL) {
+		misc_print_required_parm("--name/-N");
+		return EXIT_FAILURE;
+	}
+
+	rc = keystore_convert_key(g.keystore, g.name, g.key_type, g.noapqncheck,
+				  g.force, g.pkey_fd, &g.cca);
+
+	return rc != 0 ? EXIT_FAILURE : EXIT_SUCCESS;
+}
+
+/*
+ * Command handler for 'convert'.
+ *
+ * Converts secure keys from one key type to another
+ */
+static int command_convert(void)
+{
+	if (g.key_type == NULL) {
+		misc_print_required_parm("--key-type/-K");
+		return EXIT_FAILURE;
+	}
+	if (strcasecmp(g.key_type, KEY_TYPE_CCA_AESCIPHER) != 0) {
+		warnx("Secure keys can only be converted into key type %s",
+		      KEY_TYPE_CCA_AESCIPHER);
+		return EXIT_FAILURE;
+	}
+
+	if (g.pos_arg != NULL)
+		return command_convert_file();
+	else
+		return command_convert_repository();
+
+	return EXIT_SUCCESS;
+}
+
 /**
  * Opens the keystore. The keystore directory is either the
  * default directory or as specified in an environment variable
@@ -1777,6 +2126,9 @@ int main(int argc, char *argv[])
 		case 'r':
 			g.run = 1;
 			break;
+		case 'K':
+			g.key_type = optarg;
+			break;
 		case 'F':
 			g.force = 1;
 			break;
@@ -1867,8 +2219,7 @@ int main(int argc, char *argv[])
 	}
 
 	if (command->need_cca_library) {
-		rc = load_cca_library(&g.lib_csulcca, &g.dll_CSNBKTC,
-				      g.verbose);
+		rc = load_cca_library(&g.cca, g.verbose);
 		if (rc != 0) {
 			rc = EXIT_FAILURE;
 			goto out;
@@ -1887,8 +2238,8 @@ int main(int argc, char *argv[])
 	rc = command->function();
 
 out:
-	if (g.lib_csulcca)
-		dlclose(g.lib_csulcca);
+	if (g.cca.lib_csulcca)
+		dlclose(g.cca.lib_csulcca);
 	if (g.pkey_fd >= 0)
 		close(g.pkey_fd);
 	if (g.keystore)
